<?php

namespace Drupal\ia_translations\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\language\Entity\ConfigurableLanguage;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

class TranslationForm extends FormBase {

  protected $entityTypeManager;
  protected $entity;

  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  public function getFormId() {
    return 'ia_translation_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL) {
    $route_match = \Drupal::routeMatch();
    $entity = $route_match->getParameter('node') ?: $route_match->getParameter('taxonomy_term');

    if (!$entity) {
      throw new \InvalidArgumentException('Entity not found in route parameters.');
    }

    $this->entity = $entity;

    $fields = \Drupal::service('entity_field.manager')->getFieldDefinitions($this->entity->getEntityTypeId(), $this->entity->bundle());


    $form['translatable_fields'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Translatable Fields'),
      '#options' => [],
      '#required' => TRUE,
    ];
    foreach ($fields as $field_name => $field_definition) {
      if ($field_definition->isTranslatable() && in_array($field_definition->getType(), ['string', 'text_with_summary', 'string_long'])) {
        $form['translatable_fields']['#options'][$field_name] = $field_definition->getLabel();
        $form['translatable_fields']['#default_value'][] = $field_name;
      }
    }

    $original_language = $this->entity->getUntranslated()->language()->getId();
    $languages = ConfigurableLanguage::loadMultiple();
    $language_options = [];
    foreach ($languages as $langcode => $language) {
      if (!in_array($langcode, [$original_language, 'und', 'zxx'])) {
        if (!$this->entity->hasTranslation($langcode)) {
          $language_options[$langcode] = $language->label() . ' (New translation)';
        } else {
          $language_options[$langcode] = $language->label() . ' (Overwrite current translation)';
        }
      }
    }
    $form['languages'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Languages to Translate'),
      '#options' => $language_options,
      '#required' => TRUE,
      '#default_value' => array_keys($language_options),
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $entity_id = $this->entity->id();
    $entity_type = $this->entity->getEntityTypeId();

    $batch = [
      'title' => $this->t('Creating translations...'),
      'operations' => [],
      'finished' => '\Drupal\ia_translations\Form\TranslationForm::batchFinished',
    ];

    $selected_fields = array_filter($form_state->getValue('translatable_fields'));
    $selected_languages = array_filter($form_state->getValue('languages'));

    foreach ($selected_fields as $field_name) {
      foreach ($selected_languages as $langcode) {
        $batch['operations'][] = [
          '\Drupal\ia_translations\Form\TranslationForm::processTranslation',
          [$entity_type, $entity_id, $field_name, $langcode],
        ];
      }
    }

    batch_set($batch);
  }

  public static function processTranslation($entity_type, $entity_id, $field_name, $langcode, &$context) {
    $entity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($entity_id);
    $original_language = $entity->getUntranslated()->language()->getId();

    if (!$entity->hasTranslation($langcode)) {
      $translated_entity = $entity->addTranslation($langcode);
    } else {
      $translated_entity = $entity->getTranslation($langcode);
    }

    $original_field = $entity->getTranslation($original_language)->get($field_name);
    $original_value = $original_field->value;
    $new_value = self::translate_text($original_value, $langcode);

    if ($new_value['content']) {
      $new_value = $new_value['content'];
      if (!empty($original_field->format)) {
        $original_format = $original_field->format;
        $translated_entity->set($field_name, [
          'value' => $new_value,
          'format' => $original_format,
        ]);
      } else {
        $translated_entity->set($field_name, $new_value);
      }
      $translated_entity->save();
    }

    $context['results'][] = ['entity_id' => $entity_id, 'field_name' => $field_name, 'langcode' => $langcode];
  }

  public static function translate_text($original_value, $langcode) {
    $language_manager = \Drupal::service('language_manager');
    $language = $language_manager->getLanguage($langcode);
    $language_name = $language->getName();

    $prompt = "Correct the spelling, grammar, and translate the text to \"$language_name\". I want you to return only the translated text, do not provide any explanations. Maintain the HTML or Markdown formatting if the original text has that format.\n---TEXT START---\n$original_value\n---TEXT END---";

    $config = \Drupal::config('prompt_chatgpt.settings');
    $request_result = \Drupal::service('chatgpt.prompt_service')->api_prompt_request($prompt, $config, FALSE);

    return $request_result;
  }

  public static function batchFinished($success, $results, $operations) {
    if ($success) {
      \Drupal::messenger()->addMessage(t('Translations have been created.'));
    } else {
      \Drupal::messenger()->addMessage(t('An error occurred.'), 'error');
    }
  }
}
